% AVERAGE  computes the background image for a
%          certain exposure time and binning by interpolation from the
%          different exposure times recorded;
%
%  SYNOPSIS
%    out = average (format,texp,bin,inputdir)
%
%  PARAMETERS
%  format:    'dark' or 'white'
%  texp:      exposure time
%  bin:       binning
%  inputdir:  directory where the images are located; the images
%             inputdir=0 selects directory through user interface
%
% NOTE:    Slope and offset image name format should be 'dark_current_bin%d' and
%          'bias_bin%d' for dark or 'white_slope_bin%d' and 'white_offset_bin%d'
%          for white reference images. Those images can be calculated via
%          function lsq.m

% (C) Copyright 2009
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
% Milos Vulovic & Bernd Rieger

function out=average(varargin)


d = struct('menu','TEM CCD',...
           'display','Background image',...
           'inparams',struct('name',       {'q','dir1','texp','bin'},...
                             'description',{'Name Format','Input directory','Exposure Time','Binning'},...
                             'type',       {'option','indir','array','array'},...
                             'dim_check',  {0,0,-1,-1},...
                             'range_check',{{'dark','white'},'','R+','N+'},...
                             'required',   {1,1,1,0,},...
                             'default',    {'dark','',0,1}...
                            ),...
           'outparams',struct('name',{'out'},...
                              'description',{'Output image'},...
                              'type',{'image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [q,dir1,texp,bin] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end


if dir1==0
    dir1=uigetdir('/home', 'Select directory where the reference images are' );
end

switch q
    case 'dark'
        %averaging of the dark reference images
        % Calculate an average dark image Ibg for exposure texp
        bias = 'bias_bin%d';
        dcur = 'dark_current_bin%d';
        % Read in the bias and dark current
        fprintf('Reading bias image\n');
        s1=[dir1 filesep sprintf(bias,bin)];
        Ibias=readim(s1);
        clear s0;
        fprintf('Reading dark current image\n');
        Idcur=readim([dir1 filesep sprintf(dcur,bin)]);
        Ibg=Ibias+(texp*Idcur);
        out=Ibg;
        % Write out the average background image for given int time and binning
        p0='av_dark_bin%d_Exp%05.2f';
        p2=[dir1 filesep sprintf(p0,bin,texp)];
        writeim(dip_image(Ibg,'sfloat'),p2,'tiff','no');
        fprintf('Writing out %s\n',p2);
        
    case 'white'
        % Calculate an average white image
        sl='white_slope_bin%d';
        I0='white_offset_bin%d';
        
        % Read in the offset and slope
        fprintf('Reading offset image\n');
        s0=sprintf(I0,bin);
        s1=[dir1 filesep s0];
        Iwhite0=readim(s1);
        clear s0;
        fprintf('Reading slope image\n');
        s0=sprintf(sl,bin);
        s1=[dir1 filesep s0];
        slope=readim(s1);
        Iwhite=Iwhite0+(texp*slope);
        out=Iwhite;
        % Write out the average background image for given int time and binning
        
        k0='av_white_bin%d_Exp%05.2f';
        k2=[dir1 filesep sprintf(k0,bin,texp)];
        
        writeim(dip_image(Iwhite,'sfloat'),k2,'tiff','no');
        fprintf('Writing out %s\n',k2);
    otherwise
        error('unknown type of reference image.');
end
