%CORR_RAW corrects a raw image for the bias and gain;
%         makes a gain scaled image which contains information about
%         pixels variation sensitivity
%
%  SYNOPSIS
%   [corr,scaled] = corr_raw(ImageIn,dir,texp,bin,dc)
%  
%PARAMETERS
%ImageIn  raw image to be corrected
%  texp: exposure time
%  bin:  binning
%  dir1: directory + name of the calibration image  
%  dc:   'nodarkcurrent' do not take into account dark current since there are only 
%        reference images with one inegration time available 
%        'darkcurrent' takes value of the dark current into account while correcting the
%        image
%
%  out(1) corrected image
%  out(2) gain scaled image 
% 
%  NOTES 
%    Images should be first cleaned up from outliers with function remout.m,
%    and find average background 'av_dark_bin%d_Exp%05.2f' and average 
%    white image 'av_white_bin%d_Exp%05.2f' via function average.m or
%    averagesame.m; the dark current image name format should be 'dark_current_bin%d'
%    (it can be calculated via function lsq.m)

% (C) Copyright 2009
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
% Milos Vulovic & Bernd Rieger      

function varargout=corr_raw(varargin)

d = struct('menu','TEM CCD',...
           'display','Correct a raw image',...
           'inparams',struct('name',       {'ImageIn','dir1','texp','bin','q'},...
                             'description',{'Input image','Input directory','Exposure Time','Binning','Dark current image included?'},...
                             'type',       {'image','indir','array','array','option'},...
                             'dim_check',  {[],0,-1,-1,0},...
                             'range_check',{[],'','R+','N+',{'nodarkcurrent','darkcurrent'}},...
                             'required',   {1,1,1,0,0},...
                             'default',    {'a','',0,1,'nodarkcurrent'}...
                            ),...
           'outparams',struct('name',{'corrected_image','scaled_image'},...
                              'description',{'Corrected image','Gain scaled image'},...
                              'type',{'image','image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end
try
   [ImageIn,dir1,texp,bin,q] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end





crop=8;

Ibg =   readim([dir1 filesep sprintf(['av_dark_bin%d_Exp%05.2f'],bin,texp)]);
%readin average white
Iwhite = readim([dir1 filesep sprintf(['av_white_bin%d_Exp%05.2f'],bin,texp)]);

clear s0

tmp=Iwhite-Ibg;
Isc=tmp/mean(tmp);
clear tmp;
% Calculated Corrected the image
fprintf('Global correction image\n');
Icorr0=dip_image((ImageIn-Ibg)/Isc);
% Identify unreliable pixels

if strcmp(q,'nodarkcurrent')
    Imask=(Isc <0.2 | Isc > 2 );
else
    Idcur=readim([dir1 filesep sprintf('dark_current_bin%d.tif',bin)]);
    Imask=(Isc <0.2 | Isc > 2 | Idcur>20);
end

clear Idcur
clear Ibias
% Mask out the borders
Imask(0:crop,:)=0;
Imask(:,0:crop)=0;
Imask(end-crop:end,:)=0;
Imask(:,end-crop:end)=0;

%cosmetic correction to make a visualy 'better' image
% Calculate nearest distance between masked pixels and non-masked pixels
dmap=dt(Imask);
pix = find(Imask);
sz  = size(Imask);
Icorr1=Icorr0;
fprintf('Found %d bad pixels\n',sum(Imask));
for ii=pix'
   ds = ceil(double(dmap(ii)))+3;
   [x,y]=meshgrid(-ds:ds);
   %square ROI around pixel
   ind = sz(2).*y+x+ii;
   %pixel in mask or not
   mask= (Imask(ind)==0);
   %retrieve image values
   val = Icorr0(ind);
   %throw away values out of mask
   val=val(mask);
   % get median value
   new0 =median(val);
   % add some noise
   std0=std(val);
   newval=new0+randn(1,1)*std0;
   Icorr1(ii)=newval;
end
% corrected image output
fprintf('Converting image to integers\n');
Icor=dip_image(Icorr1,'uint16');

varargout{1}=Icor;
if nargout>1; varargout{2}=Isc; 
end












  
