%DQE calculates Detector Quantum Efficiency of a CCD detector
%
% SYNOPSIS:
%  out = dqe(stack, avback, convfact, mtf)
%
% PARAMETERS
%  stack:    stack of uniformilly illuminated corrected images (at least 2)
%  convfact: conversion efficiency in ADU/e-
%  avback:   average background image
%  mtf:      modulation transfer function curve (calculate with
%            mtf_edge or mtf_edgeAperture and needs to be saved after running them!)
%
%  out:   dqe curve of the detector
%
%  NOTES: It is supposed that radius_lib directory (which facilitates the angular averaging)
%         is in the same directory as the functions. If this is not the case, change the line 101 of the code
%         (instead pwd use the directory where you put radius_lib)
                                 
% (C) Copyright 2009
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
% Milos Vulovic & Bernd Rieger      


function out= dqe(varargin)

d = struct('menu','TEM CCD',...
   'display','Detector Quantum Efficiency ',...
   'inparams',struct('name',       {'stack','back0','convfact','mtfwhole'},...
         'description',{'Input image','Background image','Conversion factor','MTF array'},...
         'type',       {'image','image','array','array'},...
         'dim_check',  {0,0,0,-1},...
         'range_check',{[],[],'N+','R'},...
         'required',   {1,1,1,1},...
         'default',    {'a','b',1,0}...
        ),...
   'outparams',struct('name',{'out'},...
                      'description',{'DQE'},...
                      'type',{'array'}...
                      )...
  );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [stack,back0,convfact,mtfwhole] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

c=2;
offset=1;
offplot=2;
sz2=size(stack);
n=sz2(3);
if n < 2
    error('Number of images should be 2 at least')
end
%tmpim = ex_slice(stack,0);
tmpim = squeeze(stack(:,:,0));
sz0=size(tmpim);
% ranx= sz0(1)/4:sz0(1)/2-1;
% rany= sz0(2)/4:sz0(2)/2-1;

if sz0(1)>= 512 
    ranx=1:512; 
    rany=ranx;
    radiusstep = 2;
elseif sz0(1) <512 && sz0(1) >= 256
    ranx=1:256; 
    rany=ranx;
    radiusstep = 1;
else
    error(' Image must be larger than 256x256')
end



%processsize = camsize;%if I want to process the whole image
b_c = 1; %binning of an acquired image
b_c2 =1; %additional binning in the real space for fastening the calculation
b_f  = 1; %additional binning in the frequency space for make it faster
ellipR = 1; %parameters of ellipse ratio between long and short axis
ellipang = 0; %angular direction of the main axis of the ellipse.
processsize=length(ranx)/(b_c2*b_f);
%%% building the averaging 'gaussian cone-cube'

sigmanul = 2.5;
sigmaR = 0.9;
dirradlib=[pwd filesep 'radius_lib'];
radfile = strcat([dirradlib filesep 'astigR'],num2str(ellipR),'astigang',num2str(ellipang),'_rad_step',num2str(radiusstep),'_max',...
    num2str(processsize),'_sig',num2str(sigmaR),'to',num2str(sigmanul),'.mat');
if exist (radfile,'file') 
    load(radfile)
else
     [radius_range, radius] = mk_gauss_cone_cube(radiusstep, processsize, ellipR, ellipang, sigmanul, sigmaR);
    save( radfile , 'radius_range', 'radius' )
end

dark=cut(back0,length(ranx));
szmtf = length(mtfwhole);
mtf1 = mtfwhole(szmtf/2+1:szmtf);
mtf =  double( resample(mtf1, length(ranx)/szmtf/radiusstep));


%averaging nnps detector
for kk=1:n-1
    fprintf('processing image number %2d\n',kk);
%     in1=ex_slice(stack,kk-1);
%     in2=ex_slice(stack,kk);
    in1=squeeze(stack(:,:,kk-1));
    in2=squeeze(stack(:,:,kk));
    noise1=cut(in1,length(ranx))-dark;
    noise2=cut(in2,length(ranx))-dark;
    ma=mean(noise1);
    mb=mean(noise2);
    noisediff0=(ma+mb)*((noise1/ma)-(noise2/mb))/(sqrt(2)*2*convfact);
    sz=size(noise1);
    dose=mean(noise1+noise2)/(2*convfact);
    x=1:sz(1);
    y=1:sz(2);
    %window
    wx=sin(pi*x/length(x));
    wy=sin(pi*y/length(y));
    w=dip_image(wy'*wx);
    noisediff =(noisediff0- mean(noisediff0))*w;
    moduo= abs(ft(noisediff));
    pspectrum=4*moduo.^2;
    %angular averaging
    display('angular averaging');
    b=pspectrum;
    nps(kk,:) = radav( b, radius );
    nnps0(kk,:)=nps(kk,:)/dose;
end
nnps = squeeze(mean(nnps0,1));
nnps=nnps(offset+1:end);
dqe1 = mtf(offset+1:end).^2 ./(nnps); 
out = dqe1;
xran = (offplot+c:processsize/2/radiusstep)/(processsize/2/radiusstep);
plot(xran, dqe1(offplot+c-offset :processsize/2/radiusstep-offset),'--b' )
xlabel('spatial frequency [Nyquist]')
ylabel('DQE')
niqa = dqe1(round(processsize/4/radiusstep));
s=sprintf('0.5 Nq  %0.2g\n',niqa);
h1=axis;
text (0.65*h1(2),0.6*h1(4),s,'FontSize',10)
