%EF_GAIN calculates average variance and average intensity within the 
%        stacks of uniformly illuminated images with the different exposure times;
%        plots the effective detector gain
%
% SYNOPSIS:
%  [av, vr] = ef_gain(inputdir,texp,bin,n,ports)
%
% PARAMETERS:
% input:
%  inputdir: Directory where the white images are stored; 
%             inputdir=0 selects directory through user interface
%  texp:     Exposure times (at least three)
%  bin:      Binning
%  n:        Number of images with the same exposure time 
%  ports:    Number of readout ports 1 or 4
%
% output: 
%     av: Mean of the intensities over selected pixels
%    var: Mean of the variance over selected pixels   
%
% NOTES: Raw images should be first cleared from outliers with function 
%        remout.m
%        Image name format ('dezinger_Bin%d_Exp%05.2f_dark%02d' for 
%        the darks or 'dezinger_Bin%d_Exp%05.2f_white%02d' for the white references)

% (C) Copyright 2009
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
% Milos Vulovic & Bernd Rieger      

function out = ef_gain(varargin)

d = struct('menu','TEM CCD',...
           'display','Effective gain',...
           'inparams',struct('name',       {'dir0','texp', 'bin', 'n', 'q'},...
                             'description',{'Input directory ','Exposure Time','Binning','Number of Images', 'Number of readout ports'},...
                             'type',       {'indir','array','array','array','array'},...
                             'dim_check',  {0,-1,-1,0,0},...
                             'range_check',{'','R+','N+','N+','N+'},...
                             'required',   {1,1,0,0,0},...
                             'default',    {'',0,1,10,1}...
                            ),...
          'outparams',struct('name',{'intensity','variance'},...
                              'description',{'Mean intensity','Mean Variance'},...
                              'type',{'array','array'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [dir0,texp, bin, n, q] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end


if dir0==0
dir0=uigetdir('/home', 'Select directory where the reference images are located' );
end
if length(texp)<3
	error('Number of exposure times should be at least 3');
end
if q ~=1 && q~=4
   error('Must be 1 or 4 readout ports.');
end
based='dezinger_Bin%d_Exp%05.2f_dark%02d';
baseg='dezinger_Bin%d_Exp%05.2f_white%02d';

crop=8; %exclude the border of a few pixels
thres=0.01; %threshlod

nexp=length(texp);
ave=zeros(nexp,1);
vre=ave;

cnt=0;
fn=[];

for t1=texp
    for n1=1:n;
        cnt=cnt+1;
        s1=sprintf(based,bin,t1,n1);
        fn{cnt}=[dir0 filesep s1];
        fprintf('file %s\n',fn{cnt});

    end
end
tmpim = readim(fn{1});
szfull=size(tmpim);

if q==1 %if the camera has one readout port
    nparts=1;
    sz0 = szfull/nparts;
    quadrant=1;
 sz=[sz0(1)/bin-crop, sz0(2)/bin-crop];
 X=[crop];
 Y=[crop];
 all1=newim(sz(1)-crop,sz(2)-crop,n);
 all2=newim(sz(1)-crop,sz(2)-crop,n);
 all=newim(sz(1)-crop,sz(2)-crop,n,'single');
elseif q==4
    nparts=2; % 4 readout ports
    sz0 = szfull/nparts;

    quadrant=4;

    sz=[sz0(1)/bin, sz0(2)/bin];
 X=[crop,sz(1),crop,sz(2)];
 Y=[crop,crop,sz(1), sz(2)];
all1=newim(sz(1)-crop,sz(2)-crop,n);
all2=newim(sz(1)-crop,sz(2)-crop,n);
 all=newim(sz(1)-crop,sz(2)-crop,n,'single');
end

figH=figure;
out=zeros(quadrant,n,2);
for ii=1:quadrant
    fprintf('Working on quadrant %d\n',ii);
    cnt=0;
    for expT=texp
             %read in white images
        for j=1:n
            s0=sprintf(baseg,bin,expT,j);
            s1=[dir0 filesep s0];
            %fprintf('Reading image %s\n',s0);
            I=readim(s1);
            Isub=I(X(ii):X(ii)+sz(1)-crop-1,Y(ii):Y(ii)+sz(2)-crop-1);
            all1(:,:,j-1)=Isub;
        end
        clear I Isub
        
            % read in dark referencies
        for j=1:n
            s0=sprintf(based,bin,expT,j);
            s1=[dir0 filesep s0];
            fprintf('Reading image %s\n',s0);
            I=readim(s1);
            Isub=I(X(ii):X(ii)+sz(1)-crop-1,Y(ii):Y(ii)+sz(2)-crop-1);
            all2(:,:,j-1)=Isub;
        end
        clear I Isub
      
        %average dark
        all2m=mean(all2,[],3);
        all2mr=repmat(all2m,[1,1,n]);
        
        %stack of n white images with subtracted dark references 
        all=all1-all2mr;
        oneim=squeeze(all(:,:,1));
        
        % find the median of first image and exclude pixels out of certain
        % range
        Imed= median(oneim);
        sel=(oneim > Imed*(1-thres)) & (oneim < Imed*(1+thres));
        clear Imed oneim
        
        fprintf('Number of pixels selected for exposure time %8.2f : %d\n',expT,sum(sel));
        selall=repmat(sel,[1,1,n]);
        t1=squeeze(mean(all,selall,3));
        av=double(t1(sel));
        t2=squeeze(var(all,selall,3));
        vr=double(t2(sel));
     
        
%         %plots indivual dots:   
%         figure(ii)
%         plot(av,vr,'.r');
%         hold on
%         drawnow
%         figure(ii+quadrant)
%         plot(median(av),median(vr),'+b');
%         hold on
%         drawnow
%         pause(2);
        cnt=cnt+1;
        out(ii,cnt,1)=mean(mean(av));
        out(ii,cnt,2)=mean(mean(vr));
        
    end         
        %% ploting       
        ave(1:nexp)= out(ii,1:nexp,1);
        vre(1:nexp)= out(ii,1:nexp,2);
     
        figure(figH);
        hold on
        if q==1;
            plot(ave,vre,'+b');
        else
        subplot(2,2,ii)
        plot(ave,vre,'+b');
        end

        hold on
    
     %for linear curve fit
         p=polyfit(ave,vre,1);
         xmin=min(ave);
         xmax=max(ave);
      
      Xlin=linspace(xmin,xmax,100);
      Ycalc=p(1)*Xlin+p(2);
    
    s=sprintf('     Gain: %0.2g [ADU/pe-]\n ', p(1));   
    figure(figH)
    hold on
    if q==1;
            plot(Xlin,Ycalc,'-r');
            sp=('Effective gain');
        else
        subplot(2,2,ii)
        plot(Xlin,Ycalc,'-r');
        sp=sprintf('Quadrant %1d', ii);
    end   
    title(sp)
    hold on
   
    h=axis;
    text (0.8*h(1),0.6*h(4),s,'FontSize',8)
    xlabel('Intensity [ADU]')
    ylabel('Variance [ADU^2]')
    drawnow
    %correlation coeficient
    r=corrcoef(ave',vre');
end

hold off

