%LSQ makes a linear least square fit of the intensity against 
% integration (exposure) times of each pixel in a stack of images; 
% computes offset and slope of the fit
%
% SYNOPSIS
%  [slope,offset] = lsq(format,inputdir,expT,binning,n,w)
%
% PARAMETERS
%  format:   'dark' or 'white'
%  inputdir: Directory where the files are located
%  expT:     Row vector with integration (exposure) times 
%            (at least 3 values)
%  binning:  Binning of the acquired images
%  n:        Number of images with the same exposure time
%  w=1        writes the offset a and slope image which are used later for
%            calculation of average dark or white reference image
% DEFAULTS:  w = 0
%  
%  slope=out(1) image with the values of the slopes from leanear least square fit
%         for each pixel 
%  offset=out(2) image with the values of intersections of the fit with intensity 
%         axis for each pixel
%  
% NOTE : outliers from the images should be first excluded with function
%        remout.m  

% (C) Copyright 2009
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences      Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
% Milos Vulovic & Bernd Rieger      

function varargout = lsq(varargin)

d = struct('menu','TEM CCD',...
           'display','Offset and slope estimation',...
           'inparams',struct('name',       {'s0','dir0','expT','binning','n','w'},...
                             'description',{'Name Format','Input directory','Exposure Time','Binning','Number of Images','Write images?'},...
                             'type',       {'option','indir','array','array','array','boolean'},...
                             'dim_check',  {0,0,-1,-1,0,0},...
                             'range_check',{{'dark','white'},'','R+','N+','N+',''},...
                             'required',   {1,1,1,0,0,0},...
                             'default',    {'dark','',0,1,10,0}...
                            ),...
           'outparams',struct('name',{'slope','offset'},...
                              'description',{'Slope','Offset'},...
                              'type',{'array','array'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end
try
   [s0,dir0,expT,binning,n,w] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

switch s0
   case 'dark'
      s0 = 'dezinger_Bin%d_Exp%05.2f_dark%02d';
      sm=0;
   case 'white'
      s0 = 'dezinger_Bin%d_Exp%05.2f_white%02d';
      sm=1;
   otherwise
      error('Unknown filename format.')
end


if length(expT)<3
	error('Number of exposure times should be at least 3');
end


nr=1:n;
bin=binning;
cnt=0;
fn=[];
T=[];
for t1=expT
    for n1=nr;
        cnt=cnt+1;
        s1=sprintf(s0,bin,t1,n1);
        fn{cnt}=[dir0 filesep s1];
        fprintf('file %s\n',fn{cnt});
        T=[T,t1];
    end
end
%dir0 = uigetdir('/home', 'Select directory where the images with different exposure time are' );

tmpim = readim(fn{1});
szfull=size(tmpim);
nparts=2;

sz = szfull/nparts;
a = newim(szfull);
b = newim(szfull);

jj=1;
for partx = [0:nparts-1]
    for party = [0:nparts-1]
        fprintf(' Part %d/%d\n',jj,nparts*nparts);jj=jj+1;
        roi = sz .* [partx , party];
        tmpim = readim(fn{1}); 

        tmpim = tmpim(roi(1):(roi(1)+sz(1)-1),roi(2):(roi(2)+sz(2)-1));
        I=reshape(tmpim,[1,prod(sz)]);
        for ii=2:length(T)
            fprintf('%02d/%02d read %s\n',ii,length(T),fn{ii});
            tmpim = readim(fn{ii});
            tmpim = tmpim(roi(1):(roi(1)+sz(1)-1),roi(2):(roi(2)+sz(2)-1));
            I=cat(1,I,reshape(tmpim,[1,prod(sz)]));
        end
        I=double(I);

        fprintf(' matrix inversion\n')
        M = [T;ones(size(T))]';
        res = inv(M'*M)*M'*I';
        clear M I
        a(roi(1):(roi(1)+sz(1)-1),roi(2):(roi(2)+sz(2)-1))= reshape(dip_image(res(1,:)),sz);
        b(roi(1):(roi(1)+sz(1)-1),roi(2):(roi(2)+sz(2)-1))= reshape(dip_image(res(2,:)),sz);
    end
end

if  ~exist('w') || w==1
    if sm==0
    %darks
     p0='dark_current_bin%d';
     k0='bias_bin%d';
    else
    %%whites
     p0='white_slope_bin%d';
     k0='white_offset_bin%d';
     end
     p2=[dir0 filesep sprintf(p0,bin)];
     k2=[dir0 filesep sprintf(k0,bin)];

    % Write out the dark current and bias images for different binnings
    writeim(dip_image(a,'sfloat'),p2,'tiff','no');
    fprintf('Writing out %s\n',p2);
    writeim(dip_image(b,'sfloat'),k2,'tiff','no');
    fprintf('Writing out %s\n',k2);
end
varargout{1} = a;
varargout{2} = b;


    
    
    
    
    
   
