%MTF_EDGE   MTF estimation via an image of sharp straight edge (beam stop);
%           plots mtf curve
% SYNOPSIS:
%  [mtf, lsf, psf] = mtf_edge(image_in, noisesuppression)
%
% PARAMETERS:
%          image_in: selection of an image (or stack) of the beam stop 
%                    under uniform illumination
%  noisesuppression: filters the noise on the bright side
%  DEFAULTS:
%  noisesuppression =0
%
% NOTES: image_in has to be normalized (average bright background =1,
%        average dark background close to zero) 
%        MTf is not automatically saved (check 'save' and 'load')

% (C) Copyright 2009
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
% Bernd Rieger & Milos Vulovic 2009

function varargout = mtf_edge(varargin)


d = struct('menu','TEM CCD',...
           'display','MTF estimation (edge method - beamstop)',...
           'inparams',struct('name',       {'imagein', 'noisesup'},...
                             'description',{'Input image','Noise suppression'},...
                             'type',       {'image',      'boolean'},...
                             'dim_check',  {0,            0},...
                             'range_check',{[],           []},...
                             'required',   {1,            0},...
                             'default',    {'a',          0}...
                            ),...
           'outparams',struct('name',{'mtfu','lsfu','psfu','out'},...
                              'description',{'MTF','LSF','PSF','rotated image'},...
                              'type',{'image','image','image','image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end
try
   [imagein, noisesup] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

sz=size(imagein);
d_sig= 1;
samp = 8; 

if length(sz)>2    
    n=sz(3);
    imagein=cut(imagein, [sz(1)-mod(sz(1),4), sz(2)-mod(sz(2),4), n]);
else
    n=1;
    imagein=cut(imagein, sz-mod(sz,4));
end


for kk=1:n
    %image_in=ex_slice(imagein,kk-1);
    sz=size(imagein);
    if length(sz)>2
        image_in=squeeze(imagein(:,:,kk-1));
    else
        image_in=imagein;
    end
        sz_img = size(image_in);

        % extracting the edge
        gm = gradmag(image_in,3);
        mask = threshold(gm);

        g = gradient(image_in,1);
        gxx = mean(g{1}*g{1}, mask);
        gxy = mean(g{1}*g{2}, mask);
        gyy = mean(g{2}*g{2}, mask);
        [ang] = double(dip_symmetriceigensystem2(gxx,gxy,gyy,{'phi1'}));
        %o = structuretensor(image_in,1,5,{'orientation'});
        %o = abs(o);
        %ang=double(mean(o,mask));

        % placing the edge exactly perpendicular to images boundaries
        out = rotation(dip_image(image_in,'sfloat'),pi-ang,1,'4-cubic');
        out = cut(out,sz_img-4);
        m = out<=0;
        outmedi = medif(out);
        out(m) = outmedi(m);

        sz = size(out);
        range = round([sz(1)*.25 : sz(1)*.75]);
        % line spread function
        lsf = squeeze(mean(out(:,range),[],2));
        lsfu = resample(lsf,samp,0, '4-cubic');
        dipsetpref('DerivativeFlavour','finitediff');
        %point spread function
        psfu = dx(lsfu); %psfu = dip_finitedifferenceex(lsfu, 1, 1, 0);
        psfu = resample(psfu,1/samp, 0, '4-cubic');
        dipsetpref('BoundaryCondition', '0_order');
        psf = dx(lsf, d_sig);
        dipsetpref('BoundaryCondition','symmetric');
        if noisesup
          % -- 'filtering out' the photon noise on the bright side --
          [bla,i]=max(lsf);
          id = length(lsf);
          if i > id/2
             psf(round(id/2+id/4):end)=0; 
             psfu(round(id/2+id/4):end)=0;
          else
             psf(0:round(id/2-id/4))=0;    
             psfu(0:round(id/2-id/4))=0;   
          end
        end
        mtf = abs(ft(psf));
        mtf = double(mtf/max(mtf));
        mtfu = abs(ft(psfu));
        mtfu = double(mtfu/max(mtfu));

                % figure
                % plot(mtf,'g--');hold on
                % plot(mtfu,'r');hold off
                % legend('direct gaussian derivative','oversampling and finite difference');
        if noisesup; title('Removal of noise on the bright side of the edge');end
        id = length(mtfu);
        id = id/2 + id/4;
        % -- return value is average over the 2 neighbours --
        if noisesup
          fprintf('0.5 Nyquist : %2.2f (direct) %2.2f (oversample) [removal of noise on the bright side]\n',mean(mtf(id-1:id+1)),mean(mtfu(id-1:id+1)));
        else
           fprintf('0.5 Nyquist: %2.2f (direct) %2.2f (oversample)\n',mean(mtf(id-1:id+1)),mean(mtfu(id-1:id+1)));
        end

   mtf1(kk,:)=double(mtfu);
   psf1(kk,:)=double(psfu);
   lsf1(kk,:)=double(lsfu);
end
mtfu=mean(mtf1,1);     
psfu=mean(psf1,1);    
lsfu=mean(lsf1,1);   
    
% -- returning the outputs --
varargout{1} = mtfu;
if nargout >1; varargout{2} = lsfu;end
if nargout >2; varargout{3} = psfu;end
if nargout >3; varargout{4} = out;end

%plots the mtf

id = length(mtfu);
x=0:id/2-1;
range=x/max(x);
hold off
%figures
plot(range,mtfu(id/2+1:id),'-b');
niqa=mean(mtfu(3*id/4-2:3*id/4));

title ('MTF edge method');
xlabel('1/(2*pixel)')
ylabel('MTF')

s=sprintf('0.5Nq  %0.2g\n',niqa);
             h1=axis;
             text (0.65*h1(2),0.6*h1(4),s,'FontSize',12)
hold off



