% R_SQUARE  Calculates the coefficient of determination to check how well the
%           estimated regression line fits the data
%
% SYNOPSIS:
%  out= r_square(nameformat,inputdir,Texp,bin,n,w)
%
% PARAMETERS
%      input:
% nameformat: 'dark' or 'white'
% n:          number of images with the same integration time
% bin:        binning
% Texp:       exposure time (at least three)
% inputdir:   directory where raw, offset and slope images are located
% w=1:        writes the r_square image
% DEFAULTS:
%    w = 0
%
%     out:   r_square image with r_square values for every pixel
%
% NOTES:
%       Name format of the acquired images: 'dezinger_Bin%d_Exp%05.2f_dark%02d' or 'dezinger_Bin%d_Exp%05.2f_white%02d'
%       Name format of the offset image:    'bias_bin%d.tif' or 'white_offset_bin%d' (calculated via lsq.m)
%       Name format of the slope image:     'dark_current_bin%d.tif' or 'white_slope_bin%d' (calculated via lsq.m)
%       binning should be 1,2,4 or 8

% (C) Copyright 2009
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
% Milos Vulovic & Bernd Rieger


function varargout=r_square(varargin)

d = struct('menu','TEM CCD',...
           'display','Linearity check (R square)',...
           'inparams',struct('name',       {'s0','dir1','Texp','bin','n','w'},...
                             'description',{'Name Format','Input directory','Exposure Time','Binning','Number of Images','Write r_square image?'},...
                             'type',       {'option','indir','array','array','array','boolean'},...
                             'dim_check',  {0,0,-1,-1,0,0},...
                             'range_check',{{'dark','white'},'','R+','N+','N+',''},...
                             'required',   {1,1,1,0,0,0},...
                             'default',    {'white','',0,1,10,0}...
                            ),...
                       'outparams',struct('name',{'r_sq'},...
                              'description',{'Output R square image'},...
                              'type',{'image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end
try
   [s0,dir1,Texp,bin,n,w] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

vr=length(Texp);
nrImages=1:n;
switch s0
    case 'dark'
        s0 = 'dezinger_Bin%d_Exp%05.2f_dark%02d';
        sm=0;
    case 'white'
        s0 = 'dezinger_Bin%d_Exp%05.2f_white%02d';
        sm=1;
    otherwise
        error('Unknown filename format.')
end
if sm==1
    sl='white_slope_bin%d';
    I0='white_offset_bin%d';
else
    sl='dark_current_bin%d';
    I0='bias_bin%d';
end

if length(Texp)<3
    error('Number of exposure times should be at least 3');
end

% Read in the offset and slope
fprintf('Reading offset image\n');
s1=[dir1 filesep sprintf(I0,bin)];
Iwhite0=readim(s1);
sz=size(Iwhite0);
fprintf('Reading slope image\n');
s1=[dir1 filesep sprintf(sl,bin)];
slope=readim(s1);
a=newim([sz(1),sz(2)]);
b=newim([sz(1),sz(2)]);
c=newim([sz(1),sz(2)]);
oneT=newim([sz(1),sz(2),n]);

for t=Texp
    Iwhite=Iwhite0+(t*slope);
    rep1=repmat(Iwhite,[1,1,n]);
    % read in raw images
    for n1=nrImages;
        p1=sprintf(s0,bin,t,n1);
        p2=[dir1 filesep p1];
        fprintf('%s\n',p2);
        Itmp=readim(p2);
        oneT(:,:,n1-1)=Itmp;
    end
    
    %unexplained part of the total variation of R^2
    diffun=(oneT-rep1)/n;
    diffsq1=diffun.^2;
    b1=squeeze(sum(diffsq1,[],3));
    b=b+b1;
    c=c+squeeze(mean(oneT,[],3));
end
mn = c/vr;

for  t1=Texp
    Iwhite1=Iwhite0+(t1*slope);
    %explained variation
    diffex=Iwhite1-mn;
    diffsq2=diffex.^2;
    a=a+diffsq2;
end

%calculation of r square
r_sq=a./(a+b);
varargout{1}=r_sq;

if  ~exist('w') || w==1
    if sm==1
    r0='r_square_white_Bin%d';
    else
    r0='r_square_dark_Bin%d';
    end
    r1=sprintf(r0,bin);
    r2=[dir1 filesep r1];
    % Write out the r square image
    writeim(r_sq,r2,'tiff','no');
    fprintf('Writing out %s\n',r2);
end







