% READOUT NOISE  Calculates the readout noise image from the stack of dark images
%                acquired with the lowest possible integration time and writes it out
%
% SYNOPSIS:
%  [out,rd]= readout_noise_pix(inputdir,texp,bin,n)
%
% PARAMETERS
%
%  n:         number of images with the same integration time
%  inputdir:  directory where the average background image and dark reference images are located
%  texp:      integration time (the lowest possible)
%  out:       output readout noise image [ADU]
%  rd:        average readnoise [ADU]
%
%  NOTES: Name format of the raw images dark images has to be 'dezinger_Bin%d_Exp%05.2f_dark%02d'
%         Name of the average background image should be in format
%         'av_dark_bin%d_Exp%05.2f' (calculated via average.m or averagesame.m)

% (C) Copyright 2009
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
%  Milos Vulovic & Bernd Rieger

function varargout =readout_noise_pix(varargin)

d = struct('menu','TEM CCD',...
           'display','Readout noise',...
           'inparams',struct('name',       {'dir1','texp','bin','n'},...
                             'description',{'Input directory','Exposure Time','Binning','Number of Images'},...
                             'type',       {'indir','array','array','array'},...
                             'dim_check',  {0,-1,-1,0,},...
                             'range_check',{'','R+','N+','N+'},...
                             'required',   {1,1,0,0},...
                             'default',    {'',0,1,10}...
                            ),...
           'outparams',struct('name',{'read','rd'},...
                              'description',{'Readout noise image','Mean readout noise'},...
                              'type',{'image','array'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end
try
   [dir1,texp,bin,n] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end
avd='av_dark_bin%d_Exp%05.2f';
% Read average dark image
fprintf('Reading average dark image\n');
s0=sprintf(avd,bin,texp);
s1=[dir1 filesep s0];
Ibg=readim(s1);
sz=size(Ibg);
nrImages=1:n;
oneT=newim([sz(1),sz(2),n]);
rawdark='dezinger_Bin%d_Exp%05.2f_dark%02d.tif';
for n1=nrImages;
    p2=[dir1 filesep sprintf(rawdark,bin,texp,n1)];
    fprintf('%s\n',p2);
    Itmp=readim(p2);
    oneT(:,:,n1-1)=Itmp;
end
rep1=repmat(Ibg,[1,1,n]);
dif=oneT-rep1;
oneT_st=std(dif,[],3);
read=squeeze(oneT_st);
r0='readout_noise_Exp%05.2f_bin%d';
r1=sprintf(r0,texp, bin);
r2=[dir1 filesep r1];
% Write out the readout image for different binnings
writeim(read,r2,'tiff','no');
fprintf('Writing out %s\n',r2);
%dipshow(read,'percentile' )
rd=mean(read);
out=read;
varargout{1}=out;
if nargout>1; varargout{2}=rd; 
end
