%REMOUT  Excludes outliers caused by X rays, cosmic
%        rays or readout electronics; writes the new images in the same directory
%        Name format of the images ('Bin%d_Exp%05.2f_dark%02d'
%        or 'Bin%d_Exp%05.2f_white%02d')
%
% SYNOPSIS:
%  out= remout(format,inputdir,expT,binnning,n)
%
% PARAMETERS
%  format:     'dark' or 'white'
%  inputdir:   Directory where the files are located
%  expT:       Row vector with exposure times
%  binning:    Row vector with binnings of the acquired images
%  n:          Number of images with the same integration time
%
%  out: output image (stack of images with excluded  outliers)
%
% NOTE: this can take >10 minutes for large images

% (C) Copyright 2009
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
% Milos Vulovic & Bernd Rieger

function oneT = remout(varargin)

d = struct('menu','TEM CCD',...
           'display','Remove Outliers',...
           'inparams',struct('name',       {'nameformat','dir0','expT','binning','n'},...
                             'description',{'Name Format','Input directory','Exposure Time','Binning','Number of Images'},...
                             'type',       {'option','indir','array','array','array'},...
                             'dim_check',  {0,0,-1,-1,0},...
                             'range_check',{{'dark','white'},'','R+','N+','N+'},...
                             'required',   {1,1,1,0,0},...
                             'default',    {'dark','',0,1,10}...
                            ),...
           'outparams',struct('name',{'oneT'},...
                              'description',{'Output image'},...
                              'type',{'image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      oneT = d;
      return
   end
end
try
   [nameformat,dir0,expT,binning,n] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

%dir0 = uigetdir('/home', 'Select directory where raw images are' );
switch nameformat
    case 'dark'
        s0 = 'Bin%d_Exp%05.2f_dark%02d';
    case 'white'
        s0 = 'Bin%d_Exp%05.2f_white%02d';
    otherwise
        error('Unknown filename format.')
end

nrImages=1:n;
sig=10;
it=2; %number of iterations of rejection
DEBUG = 0;


for bin=binning
    
    cnt=0;
    fn=[];
    
    for t1=expT
        for n1=nrImages;
            cnt=cnt+1;
            u1=sprintf(s0,bin,t1,n1);
            fn{cnt}=[dir0 filesep u1];
            fprintf('file %s\n',fn{cnt});
            
            
        end
    end
    
    tmpim = readim(fn{1});
    sz_im=size(tmpim);
    
    
    for t1=expT
        oneT=newim(sz_im(1),sz_im(2),n);
        
        fprintf('Checking binning %d at exposure Time %4.2fs\n',bin,t1);
        tt=sprintf(s0,bin,t1,0);
        t2=tt(1:end-1-it);
        zlog=sprintf('zingerstatsSig%03d.log',sig);
        logname=[dir0 filesep t2 '_' zlog];
        fclose all;
        flog=fopen(logname,'wt');
        
        % make a stack of images with the same integration time
        for n1=nrImages;
            s1=sprintf(s0,bin,t1,n1);
            s2=[dir0 filesep s1];
            %fprintf('%s\n',s2);
            try
                Itmp=readim(s2);
                oneT(:,:,n1-1)=Itmp;
                fprintf('Correctly located file %s\n',s2);
            catch
                error('file %s does not exist\n', s2);
            end
        end
        clear Itmp
        fprintf('  Computing the median\n');
        % find the median of every pixel within the stack
        med_oneT = dip_image(median(oneT,[],3));
        
        % repeat this matrix
        rep1 = repmat(med_oneT,[1,1,n]);
        
        % determine the absolute difference
        db = abs(oneT-rep1);
        
        % Sort this
        [dbsort,ind]=sort(single(db),3);
        clear db dbsort
        
        % Sort oneT array according to ind
        dip_ind=dip_image(ind,'uint8');
        
        % Do 2 iterations of rejection
        for iter=0:it-1
            Ilast=squeeze( dip_ind(:,:,end-iter) );
            fprintf('Dezinger for binning %d and exposure time %4.2fs, iteration number %d ',bin,t1,iter+1);
            fprintf(flog,'Dezinger for binning %d and exposure time %4.2fs, iteration number %d ',bin,t1,iter+1);
            
            % Make a mask where you exclude the largest deviations
            mask=newim([size(Ilast) n],'bin');
            for kk=1:n
                mask(:,:,kk-1)=squeeze(Ilast==kk);
            end
            
            oneT_mn=mean(oneT,not(mask),3);
            szmn = size(oneT_mn);
            oneT_st=std(oneT,not(mask),3);
            clear mask
            
            % Set thresholds.
            % Add and subtract an offset of 10 in case std becomes really small
            thresH=repmat(oneT_mn+10+sig*oneT_st,[1,1,n]);
            thresL=repmat(oneT_mn-10-sig*oneT_st,[1,1,n]);
            clear oneT_mn oneT_st
            
            % exclude zingers
            Isel = (oneT>thresL) & (oneT<thresH);
            clear thresH thresL
            sumzin = sum(not(Isel));
            fprintf('Number of zingers: %d\n',sumzin);
            fprintf(flog,'Number of zingers: %d\n',sumzin);
            
            %replace them with the median
            if (sumzin>0)
                fl2 = findcoord(not(Isel));
                fl = sub2lin(fl2,size(oneT));
                fl3 = [fl2(:,1:2),zeros(size(fl2,1),1)];
                indmn = sub2lin(fl3, szmn);
                orgPix = single(oneT(fl));
                repPix = uint16(med_oneT(indmn));
                oneT(fl) = repPix;
                
                if DEBUG
                    for kk=1:length(fl)
                        fprintf('Replaced outlier with value %5d at %5d %5d of image %5d with new value %8d\n',orgPix(kk),fl2(kk,1),fl2(kk,2),fl2(kk,3),repPix(kk));
                        %fprintf(flog,'Replaced outlier with value %5d at %5d %5d of image %5d with new value %8d\n',orgPix(kk),fl2(kk,1),fl2(kk,2),fl2(kk,3),repPix(kk));
                    end
                end
            end
        end
        clear repPix orgPix indmn Isel
        % Write out the dezingered images
        oneT = dip_image(oneT,'uint16');
        for n1=nrImages;
            s1=sprintf(s0,bin,t1,n1);
            s2=[dir0 filesep 'dezinger_' s1];
            fprintf('Writing image %s\n',s2);
            %writeim(slice_ex(oneT,n1-1),s2,'tiff','no');
            writeim(squeeze(oneT(:,:,n1-1)),s2,'tiff','no');
        end
        
        fclose(flog);
    end
end


